/* (C) Copyright 1984,85,86,87 Walter L. Peacock   All Rights Reserved    */
/* b t r d e l e t . c                                    */

/*---------------------------------------------------------------------------- 
 * btrdelet() is called to delete a key from the index page.  It invokes 
 * other lower-level B+tree functions as needed to maintain the B+tree,
 * i.e. adjstack(), deletrec(), deleteky(), cpyrtbro(), cpyltbro(),
 * cmptnode().
 *---------------------------------------------------------------------------*/

#include "cbtree.h"

int btrdelet(fddta, fdidx, idxblk, hdrloc, node, btparmst)
int fddta, fdidx, hdrloc;
long node;
BTC *btparmst;
struct btidxblk *idxblk;
{
   extern int strncmp(), strlen();
   int retcd, adjck = 0;

   /* first make sure that we have the right record   */
   if (btparmst->btloc > 0L){
      while(idxblk->btpage[hdrloc].btptr != btparmst->btloc){
         if (++hdrloc >= idxblk->cellicnt){
            if (idxblk->fwdpage > 1L) {      /* look at next node */
               adjck = 1;
               getidxr(fdidx, node = idxblk->fwdpage, idxblk,
                     btparmst->btidxlen, btparmst->btkeylen);
               hdrloc = 0;
            }
            else{               /* continuation not found. */
               return(BTNOTFND);
            }
         }
         if (strncmp(btparmst->btkey, idxblk->btpage[hdrloc].skeynme,
               strlen(btparmst->btkey)) != 0)
         {
            return(BTNOTFND);
         }
      }
      if (adjck == 1){
         /* we may have moved accross upper level nodes so adjust the stack*/
         adjstack(fdidx, idxblk, node, btparmst);
         getidxr(fdidx,node, idxblk, btparmst->btidxlen, btparmst->btkeylen);
      }
   }
   if (btparmst->btoptype == DELETE)
   {
      if (btparmst->btvarlen)
         btdelvar(fddta, idxblk->btpage[hdrloc].btptr);
      else
         deletrec(fddta, idxblk->btpage[hdrloc].btptr, btparmst->btdtalen);
   }
   /* delete record and add to freespc chain */
   deleteky(idxblk, hdrloc);    /* now delete the B-tree key cell */

   /* check for end cell. if end cell then update upper level */
   /* if no right brother then no need to update upper level  */
   if ((hdrloc == idxblk->cellicnt) && (idxblk->fwdpage > 1L))
      updilvl(fdidx, node, idxblk->btpage[hdrloc-1].skeynme, btparmst);

   if ((idxblk->cellicnt >= (btparmst->btcells >> 1)) /* LC bug */
            || (node == btparmst->btroot) ){
      /* delete occured w/o underflow, write the index record to disk */
      putidxr(fdidx, node, idxblk, btparmst);
      return(BTCALLOK);
   }

   /* since we are here, we do not have enough nodes so must copy nodes */
   /* from brother and check for underflow to maintain btree balance.      */
   /* first try to copy from right brother */
   retcd = cpyrtbro(fdidx, idxblk, btparmst);
   if(retcd == -1){ /* next try to copy from left brother */
      retcd = cpyltbro(fdidx, idxblk, btparmst);
      if(retcd == -1){ /* O.K. then, compact two nodes into one */
         retcd = cmptnode(fddta, fdidx, idxblk, btparmst);
      }
   }

   if(retcd < 1)
      return(BTERROR);

   /* write node... balanced tree has been maintained.   */
   putidxr(fdidx, node, idxblk, btparmst);
   return(BTCALLOK);  /* finally return to calling pgm. */
}

/* include this here to remove circular library references in DC */
#include "cmptnode.c"
